body {
	background-color: #ffffff;
	font-family: sans-serif;
	margin: 0;
	padding: 0;
}

#player {
	background-color: #272727;
	height: 124px;
}

#trackcover, #trackcover img {
	width: 100px;
	height: 100px;
	float: left;
}

#trackinfo {
	height: 100px;
	margin-left: 100px;
	color: #eeeeee;
}

#trackcontrol {
	background-color: #436a9c;
	height: 24px;
}

#trackcontrol .button {
	float: left;
	line-height: 24px;
	font-size: 10px;
	padding: 0px 10px;
	color: #ffffff;
}

#trackcontrol .button:hover {
	background-color: #8abc63;
	cursor: pointer;
}

#ti_title {
	font-size: 22px;
	padding: 5px 5px 5px 20px;
	font-weight: bold;
}

.trackinfo {
	font-size: 12px;
	padding: 0px 20px;
}

.tibold {
	font-weight: bold;
}

#trackprogressbar {
	background-color: white;
	width: auto;
	height: 20px;
	line-height: 20px;
	margin: 5px 20px 0px 20px;
}

.trackprogressbarlen {
	color: black;
	text-align: center;
	font-size: 12px;
	font-weight: bold;
	line-height: 20px;
	height: 20px;
}

.trackprogressbarposition {
	background-color: #aaaaaa;
	width: 100%;
	height: 20px;
	margin-top: -20px;
}

#playlistcontent table {
	font-size: 12px;
	border-collapse: collapse;
	width: 100%;
}

#playlistcontent table th {
	text-align: left;
	background-color: #436a9c;
	padding: 5px;
	color: #ffffff;
}

#playlistcontent table td {
	border-top: 1px solid #eeeeee;
	padding: 5px;
}

#playlistcontent table td div {
	white-space: nowrap;
	overflow: hidden;
}

