#!/usr/bin/env python
"""This file is automatically generated by generate_version_info
It uses the current working tree to determine the revision.
So don't edit it. :)
"""

version_info = {'branch_nick': u'main',
 'build_date': '2008-08-22 21:25:50 -0500',
 'clean': None,
 'date': '2008-08-02 01:15:31 +0200',
 'revision_id': 'info@noctus.net-20080801231531-6wufpnd7lzhhxrv6',
 'revno': 1650}

revisions = {}

file_revisions = {}



if __name__ == '__main__':
    print 'revision: %(revno)d' % version_info
    print 'nick: %(branch_nick)s' % version_info
    print 'revision id: %(revision_id)s' % version_info
